package sun.security.pkcs10;

import java.util.Collections;
import java.util.Collection;
import java.util.Enumeration;
import sun.security.util.DerOutputStream;
import java.io.OutputStream;
import sun.security.util.DerValue;
import java.io.IOException;
import sun.security.util.DerInputStream;
import java.util.Hashtable;
import sun.security.util.DerEncoder;

public class PKCS10Attributes implements DerEncoder
{
    private Hashtable<String, PKCS10Attribute> map;
    
    public PKCS10Attributes() {
        this.map = new Hashtable(3);
    }
    
    public PKCS10Attributes(PKCS10Attribute[] array) {
        this.map = new Hashtable(3);
        for (int i = 0; i < array.length; ++i) {
            this.map.put(array[i].getAttributeId().toString(), array[i]);
        }
    }
    
    public PKCS10Attributes(DerInputStream derInputStream) throws IOException {
        this.map = new Hashtable(3);
        DerValue[] set = derInputStream.getSet(3, true);
        if (set == null) {
            throw new IOException("Illegal encoding of attributes");
        }
        for (int i = 0; i < set.length; ++i) {
            PKCS10Attribute value = new PKCS10Attribute(set[i]);
            this.map.put(value.getAttributeId().toString(), value);
        }
        return;
    }
    
    public void encode(OutputStream outputStream) throws IOException {
        this.derEncode(outputStream);
    }
    
    public void derEncode(OutputStream outputStream) throws IOException {
        PKCS10Attribute[] array = (PKCS10Attribute[])this.map.values().toArray(new PKCS10Attribute[this.map.size()]);
        DerOutputStream derOutputStream = new DerOutputStream();
        derOutputStream.putOrderedSetOf(DerValue.createTag((byte)(-128), true, (byte)0), array);
        outputStream.write(derOutputStream.toByteArray());
    }
    
    public void setAttribute(String key, Object o) {
        if ((o instanceof PKCS10Attribute)) {
            this.map.put(key, (PKCS10Attribute)o);
        }
    }
    
    public Object getAttribute(String key) {
        return this.map.get(key);
    }
    
    public void deleteAttribute(String key) {
        this.map.remove(key);
    }
    
    public Enumeration<PKCS10Attribute> getElements() {
        return this.map.elements();
    }
    
    public Collection<PKCS10Attribute> getAttributes() {
        return Collections.unmodifiableCollection(this.map.values());
    }
    
    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (!(o instanceof PKCS10Attributes)) {
            return false;
        }
        Collection<PKCS10Attribute> attributes = ((PKCS10Attributes)o).getAttributes();
        PKCS10Attribute[] array = (PKCS10Attribute[])attributes.toArray(new PKCS10Attribute[attributes.size()]);
        int length = array.length;
        if (length != this.map.size()) {
            return false;
        }
        int i = 0;
        while (i < length) {
            PKCS10Attribute obj = array[i];
            String string = obj.getAttributeId().toString();
            if (string == null) {
                return false;
            }
            PKCS10Attribute pkcs10Attribute = (PKCS10Attribute)this.map.get(string);
            if (pkcs10Attribute == null) {
                return false;
            }
            if (!pkcs10Attribute.equals(obj)) {
                return false;
            }
            ++i;
        }
        return true;
    }
    
    @Override
    public int hashCode() {
        return this.map.hashCode();
    }
    
    @Override
    public String toString() {
        return this.map.size() + "\n" + this.map.toString();
    }
}
